;;; GNU Guix --- Functional package management for GNU
;;; Copyright © 2016 Danny Milosavljevic <dannym@scratchpost.org>
;;;
;;; This file is part of GNU Guix.
;;;
;;; GNU Guix is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or (at
;;; your option) any later version.
;;;
;;; GNU Guix is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.

(define-module (wip arduino)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix utils)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix packages)
  #:use-module (guix build-system cmake)
  #:use-module (guix build-system gnu)
  #:use-module (guix build-system go)
  #:use-module (guix build-system python)
  #:use-module (guix build-system ant)
  #:use-module (gnu packages)
  #:use-module (gnu packages autotools)
  #:use-module (gnu packages avr)
  #:use-module (gnu packages code)
  #:use-module (gnu packages flashing-tools)
  #:use-module (gnu packages golang)
  #:use-module (gnu packages java)
  #:use-module (gnu packages textutils)
  #:use-module (gnu packages python)
  #:use-module (gnu packages python-xyz)
  #:use-module (gnu packages batik))

(define (arduino-installer filename)
  `(lambda* (#:key outputs #:allow-other-keys)
     (let* ((out (assoc-ref outputs "out"))
            (out-share (string-append out "/share/arduino"))
            (out-share-part (string-append out-share "/" ,filename)))
      (mkdir-p out-share)
      (copy-recursively ,filename out-share-part)
      #t)))

(define-public arduinocore-avr
  (package
    (name "arduinocore-avr")
    (version "1.6.23")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/arduino/ArduinoCore-avr.git")
              (commit version)))
        (sha256
         (base32
          "03xmb45rafjs5p6ypw2k94w1wj12lhpizfjj12jrnsq5216sz94d"))
        (file-name (string-append name "-" version))))
    (build-system gnu-build-system)
    (arguments
     `(#:tests? #f                      ; No tests exist
       #:modules ((guix build utils) (ice-9 match)
                  (guix build gnu-build-system))
       #:phases
       (modify-phases %standard-phases
         (delete 'configure)
         (delete 'build)
         (replace 'install
           (lambda* (#:key outputs #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (out-share (string-append out "/share/hardware/arduino"))
                    (out-share-part (string-append out-share "/avr")))
               (mkdir-p out-share)
               (copy-recursively "." out-share-part)
               #t))))))
    (home-page "https://www.arduino.cc/")
    (synopsis "Arduino AVR core")
    (description "@code{arduinocore-avr} contains the Arduino AVR core.")
    ;; FIXME.
    (license (list license:lgpl2.1+ license:gpl3+))))

; - https://github.com/arduino/ArduinoCore-sam ?
; - https://github.com/arduino/ArduinoCore-samd 1.6.19

(define-public arduino-makefile
  (package
    (name "arduino-makefile")
    (version "1.6.0")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/sudar/Arduino-Makefile.git")
                    (commit version)))
              (sha256
               (base32
                "0flpl97d2231gp51n3y4qvf3y1l8xzafi1sgpwc305vwc2h4dl2x"))
              (file-name (string-append name "-" version ".tar.gz"))))
    (build-system python-build-system)
    (arguments
     `(#:tests? #f ; no tests exist
       #:phases
        (modify-phases %standard-phases
          (delete 'configure)
          (add-after 'unpack 'patch-paths
            (lambda* (#:key inputs outputs #:allow-other-keys)
             (let ((avr-gcc (assoc-ref inputs "avr-toolchain-5")))
              (substitute* "bin/ard-reset-arduino"
                (("#!/usr/bin/env python") "#!/usr/bin/python3"))
              (substitute* "Arduino.mk"
                (("#    => ARDUINO_DIR.*")
                   (string-append "ARDUINO_DIR = "
                                  (assoc-ref %build-inputs "arduino-libraries")
                                  "/share/arduino\n")) ; TODO: "/libraries".
                ; ; defaults to "hardware/tools/avr"
                (("#    => AVR_TOOLS_DIR.*")
                   (string-append "AVR_TOOLS_DIR = "
                                  (assoc-ref %build-inputs "avrdude")
                                  "\n"))
                (("#    => ARDMK_DIR.*")
                   (string-append "ARDMK_DIR = "
                                  (assoc-ref %outputs "out")
                                  "/share/arduino\n"))
                (("CC_NAME[ ]*=.*")
                   (string-append "CC_NAME = " avr-gcc "/bin/avr-gcc\n"))
                (("CXX_NAME[ ]*=.*")
                   (string-append "CXX_NAME = " avr-gcc "/bin/avr-g++\n"))
                (("OBJCOPY_NAME[ ]*=.*")
                   (string-append "OBJCOPY_NAME = " avr-gcc "/bin/avr-objcopy\n"))
                (("OBJDUMP_NAME[ ]*=.*")
                   (string-append "OBJDUMP_NAME = " avr-gcc "/bin/avr-objdump\n"))
                (("AR_NAME[ ]*=.*")
                   (string-append "AR_NAME = " avr-gcc "/bin/avr-ar\n"))
                (("SIZE_NAME[ ]*=.*")
                   (string-append "SIZE_NAME = " avr-gcc "/bin/avr-size\n"))
                (("NM_NAME[ ]*=.*")
                   (string-append "NM_NAME = " avr-gcc "/bin/avr-nm\n"))))))
          (delete 'build)
          (replace 'install
            (lambda* (#:key outputs #:allow-other-keys)
              (let* ((out (assoc-ref outputs "out"))
                     (out-mk (string-append out "/share/arduino"))
                     (out-doc (string-append out "/share/doc"))
                     (out-bin (string-append out "/bin"))
                     (out-man (string-append out "/share/man/man1")))
                    (mkdir-p out-mk)
                    (for-each (lambda (name)
                                (copy-file name (string-append out-mk "/" name)))
                              '("Arduino.mk" "arduino-mk-vars.md" "chipKIT.mk" "Common.mk"))
                    (mkdir-p out-doc)
                    (copy-recursively "examples" out-doc)
                    (mkdir-p out-bin)
                    (copy-file "bin/ard-reset-arduino"
                               (string-append out-bin "/ard-reset-arduino"))
                    (mkdir-p out-man)
                    (copy-file "ard-reset-arduino.1"
                               (string-append out-man "/ard-reset-arduino.1"))))))))
    (inputs
     `(("python" ,python)
       ("python-pyserial" ,python-pyserial)
       ("arduino-libraries" ,arduinocore-avr)
       ("avrdude" ,avrdude)
       ("avr-toolchain-5" ,avr-toolchain-5)))
    (synopsis "Arduino Makefile Include Files")
    (description "Allows you to build Arduino sketches using a very tiny Makefile")
    (home-page "https://github.com/sudar/Arduino-Makefile")
    (license license:lgpl2.1)))

(define-public java-jssc
  (package
    (name "java-jssc")
    (version "2.8.0")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/scream3r/java-simple-serial-connector.git")
              (commit (string-append "v" version))))
        (sha256
         (base32
          "174nfvjmbllrcd43s1vwasid6va7kqyrqj5js1ixz1qjlylnrwnw"))
        (file-name (string-append name "-" version))))
    (build-system ant-build-system)
    (arguments
     `(#:phases
       (modify-phases %standard-phases
         (replace 'build
           (lambda* (#:key outputs #:allow-other-keys)
             (invoke "g++" "-I" (string-append (getenv "JAVA_HOME") "/include")
                     "-I" (string-append (getenv "JAVA_HOME") "/include/linux")
                     "-fPIC" "-shared" "-o" "libjSSC-2.8.so"
                     "src/cpp/_nix_based/jssc.cpp")
             (chdir "src/java")
             (substitute* "jssc/SerialNativeInterface.java"
              (("static \\{")
               (string-append "static {\nSystem.load(\""
                              (assoc-ref outputs "out")
                              "/lib/java/libjSSC-2.8.so\");")))
             (invoke "javac" "jssc/SerialPort.java"
                             "jssc/SerialPortException.java"
                             "jssc/SerialPortEventListener.java"
                             "jssc/SerialPortEvent.java"
                             "jssc/SerialNativeInterface.java"
                             "jssc/SerialPortList.java"
                             "jssc/SerialPortTimeoutException.java")
             (invoke "jar" "cf" "jssc.jar"
                     "jssc/SerialNativeInterface.class"
                     "jssc/SerialPort$1.class"
                     "jssc/SerialPort$EventThread.class"
                     "jssc/SerialPort$LinuxEventThread.class"
                     "jssc/SerialPort.class"
                     "jssc/SerialPortEvent.class"
                     "jssc/SerialPortEventListener.class"
                     "jssc/SerialPortException.class"
                     "jssc/SerialPortList$1.class"
                     "jssc/SerialPortList.class"
                     "jssc/SerialPortTimeoutException.class")))
         (delete 'check)
         (replace 'install
           (lambda* (#:key outputs #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (out-lib (string-append out "/lib/java"))
                    (out-share (string-append out "/share/java")))
               (mkdir-p out-lib)
               (install-file "../../libjSSC-2.8.so" out-lib)
               (install-file "jssc.jar" out-share)
               #t))))))
    (home-page "https://www.arduino.cc/")
    (synopsis "Java serial port connector")
    (description "@code{java-jssc} provides a library to use serial ports.")
    (license license:lgpl3+)))

(define-public java-semver
  (package
    (name "java-semver")
    (version "0.9.0")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/zafarkhaja/jsemver.git")
              (commit version)))
        (sha256
         (base32
          "0652dfyrqw4pi5bm8p1y4zvl4z6vg1rpkjhcgy1k6yz93c0fhpy6"))
        (file-name (string-append name "-" version))))
    (build-system ant-build-system)
    (arguments
     `(#:jar-name "semver.jar"))
    (native-inputs
     `(("java-hamcrest-core" ,java-hamcrest-core)
       ("java-junit" ,java-junit)))
    (home-page "https://github.com/zafarkhaja/jsemver")
    (synopsis "Java semantic versioning implementation")
    (description "@code{java-semver} provides a library that implements
semantic versioning.")
    (license license:expat)))

(define-public java-jmdns
  (package
    (name "java-jmdns")
    (version "3.5.4")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/jmdns/jmdns.git")
              (commit (string-append "jmdns-" version))))
        (sha256
         (base32
          "118rw2nd9z2mvqr6jh411qy0z4prizqhgcpd4bpvfa0wps9230aj"))
        (file-name (string-append name "-" version))))
    (build-system ant-build-system)
    (arguments
     `(#:test-target "test"
       #:tests? #f ; Those need network.
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'add-manifest
           (lambda _
             (mkdir "lib")
             (call-with-output-file "lib/jmdns.manifest"
               (lambda (port)
                 (format port "Main-Class: com.strangeberry.jmdns.tools.Main")))
             #t))
         (replace 'install
           (lambda* (#:key outputs #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (out-share (string-append out "/share/java")))
             (install-file "build/lib/jmdns.jar" out-share)
             #t))))))
    (inputs
     `(("java-slf4j-api" ,java-slf4j-api)))
    (native-inputs
     `(("java-easymock" ,java-easymock)
       ("java-hamcrest-core" ,java-hamcrest-core)
       ("java-junit" ,java-junit)
       ("java-slf4j-simple" ,java-slf4j-simple)))
    (home-page "https://github.com/jmdns/jmdns")
    (synopsis "Multi-cast DNS library in Java")
    (description "This package provides a multi-cast DNS in Java.")
    (license license:asl2.0)))

(define (arduino-source version)
  (origin
    (method git-fetch)
    (uri (git-reference
          (url "https://github.com/arduino/Arduino.git")
          (commit version)))
    (sha256
     (base32
      "0cp502jf3xsrisadkrmxzylds48x60vparxm5lfcrzw27hw0qxgq"))
    (file-name (string-append "arduino-" version))
    (modules '((guix build utils)))
    (snippet
      '(begin
         ;; Delete bundled jar archives.
         (for-each delete-file (find-files "." "\\.jar$"))
         ;; Delete files for other operating systems.
         (delete-file "arduino-core/src/cc/arduino/os/windows/Win32KnownFolders.java")
         (delete-file "arduino-core/src/processing/app/macosx/Platform.java")
         (delete-file "arduino-core/src/processing/app/windows/Platform.java")
         (delete-file-recursively "build/windows")
         #t))))

(define-public arduino-libserialport
  (package
    (name "arduino-libserialport")
    (version "0.0.1")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/facchinm/libserialport.git")
              (commit "16e35cfa29ecf4292f19f07141c6b112a6848c69")))
        (sha256
         (base32
          "0045cchfwq6ajpdcq5aa5wqx1gxs720mgz5rpidbw7wwk80hmyas"))
        (file-name (string-append name "-" version "-checkout"))))
    (build-system gnu-build-system)
    (arguments
     `(#:make-flags
       '("CFLAGS=-fPIC")
       #:tests? #f ; No tests exist.
       #:phases
       (modify-phases %standard-phases
         (add-before 'bootstrap 'libtoolize
           (lambda _
             (invoke "libtoolize"))))))
    (native-inputs
     `(("autoconf" ,autoconf)
       ("automake" ,automake)
       ("libtool" ,libtool)))
    (home-page "https://github.com/arduino/libserialport")
    (synopsis "Serial port library for Arduino IDE")
    (description "This package provides a serial port library to the
Arduino IDE")
    (license license:lgpl3)))

(define-public arduino-listserialportsc
  (package
    (name "arduino-listserialportsc")
    (version "1.4.0")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/arduino/listSerialPortsC.git")
              (commit version)))
        (sha256
         (base32
          "0f0pmi2fzdsd4gqvki03m2ja66h7rzxc06py51h825sqbf4kfx5r"))
        (file-name (string-append name "-" version "-checkout"))))
    (build-system gnu-build-system)
    (arguments
     `(#:tests? #f ; No tests exist.
       #:phases
       (modify-phases %standard-phases
         (delete 'configure)
         (replace 'build
           (lambda* (#:key inputs #:allow-other-keys)
             (let ((static-library
                    (string-append (assoc-ref inputs "arduino-libserialport")
                                   "/lib/libserialport.a"))
                   (icedtea (assoc-ref inputs "icedtea")))
               (invoke "gcc" "-shared" "-fPIC"
                       "-o" "liblistSerialsj.so"
                       "-I" (string-append icedtea "/include")
                       "-I" (string-append icedtea "/include/linux")
                       "jnilib.c" static-library))))
         (replace 'install
           (lambda* (#:key outputs #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (out-lib (string-append out "/lib/java")))
               (install-file "liblistSerialsj.so" out-lib)
               #t))))))
    (inputs
     `(("arduino-libserialport" ,arduino-libserialport)))
    (native-inputs
     `(("icedtea" ,icedtea "jdk")))
    (home-page "http://www.arduino.cc/")
    (synopsis "Lists serial ports for Arduino")
    (description "This package provides a library that lists serial ports
(for Arduino).")
    (license license:lgpl3)))

(define-public java-astyle
  (package
    (inherit astyle)
    (name "java-astyle")
    (build-system cmake-build-system)
    (native-inputs
     `(("icedtea" ,icedtea "jdk")))
    (arguments
     `(#:configure-flags
       '("-DBUILD_JAVA_LIBS=1")
       #:tests? #f ; No tests exist.
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'patch-java-searcher
           (lambda* (#:key inputs outputs #:allow-other-keys)
             (setenv "JAVA_HOME" (assoc-ref inputs "icedtea"))
             (substitute* "CMakeLists.txt"
              (("/usr/lib/jvm/default-java/include")
               "$ENV{JAVA_HOME}/include")
              (("DESTINATION /usr/lib")
               (string-append "DESTINATION "
                              (assoc-ref outputs "out")
                              "/lib")))
             #t)))))))

(define-public arduino-core
  (package
    (name "arduino-core")
    (version "1.8.7")
    (source
      (arduino-source version))
    (build-system ant-build-system)
    (arguments
     `(#:tests? #f
       #:build-target "build"
       #:modules
       ((guix build utils) (ice-9 match)
        (guix build ant-build-system))
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'honor-classpath
           (lambda _
             (substitute* "arduino-core/build.xml"
              (("classpathref=\"class.path\"")
               "classpath=\"${env.CLASSPATH}\""))
             #t))
         (add-after 'honor-classpath 'patch-library-loading
           (lambda* (#:key inputs outputs #:allow-other-keys)
             ;; FIXME: Find and patch other loadLibs/mapRelatives.
             (substitute* "arduino-core/src/processing/app/Platform.java"
              (("new File\\(BaseNoGui\\.getContentFile\\(\"lib\"\\), System\\.mapLibraryName\\(\"listSerialsj\"\\)\\)")
               (string-append "new File(\""
                              (assoc-ref outputs "out")
                              "/lib/java/liblistSerialsj.so\")")))
             #t))
         (add-before 'build 'chdir
           (lambda _
             (chdir "arduino-core")
             #t))
         (replace 'install
           (lambda* (#:key inputs outputs #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (out-share (string-append out "/share/java"))
                    (out-lib (string-append out "/lib/java")))
               (install-file "arduino-core.jar" out-share)
               (mkdir-p out-lib)
               ;; Make sure that the GC sees it.
               (symlink (string-append (assoc-ref inputs "arduino-listserialportsc")
                                       "/lib/java/liblistSerialsj.so")
                        (string-append out-lib "/liblistSerialsj.so"))
               #t))))))
    (inputs
     `(("arduino-listserialportsc" ,arduino-listserialportsc)))
    (propagated-inputs
     `(("jackson-annotations" ,java-fasterxml-jackson-annotations)
       ("java-fasterxml-jackson-modules-base-mrbean"
        ,java-fasterxml-jackson-modules-base-mrbean)
       ("java-bouncycastle" ,java-bouncycastle)
       ("java-commons-codec" ,java-commons-codec)
       ("java-commons-compress" ,java-commons-compress)
       ("java-commons-exec" ,java-commons-exec-1.1)
       ("java-commons-lang3" ,java-commons-lang3)
       ("java-commons-logging" ,java-commons-logging-minimal)
       ("java-commons-net" ,java-commons-net)
       ("java-fasterxml-jackson-core" ,java-fasterxml-jackson-core)
       ("java-fasterxml-jackson-databind" ,java-fasterxml-jackson-databind)
       ("java-jmdns" ,java-jmdns)
       ("java-jsch" ,java-jsch)
       ("java-jssc" ,java-jssc)
       ("java-semver" ,java-semver)
       ("java-slf4j" ,java-slf4j-api)
       ("java-slf4j-simple" ,java-slf4j-simple)))
    (home-page "https://www.arduino.cc/")
    (synopsis "Arduino Core")
    (description "Arduino Core.")
    ;; FIXME.
    (license license:gpl2+)))

;github.com/arduino/go-properties-map

;; FIXME: Unbundle Approxidate (use from Git)
; FIXME: cannot find package "gopkg.in/mgo.v2/bson".
(define-public go-github-com-simplereach-timeutils
  (package
    (name "go-github-com-simplereach-timeutils")
    (version "1.2.0")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                     (url "https://github.com/simplereach/timeutils.git")
                     (commit (string-append "v" version))))
              (file-name (git-file-name name version))
              (sha256
               (base32
                "08mf2j0dk07c9p279ydbrjxbjf113azmq0x8vhf7jvldcgjpw3m6"))))
    (build-system go-build-system)
    (arguments
     '(#:import-path "github.com/simplereach/timeutils"))
    (synopsis "Go wrapper for Approxidate")
    (description "This package provides a Go Wrapper for Approxidate.")
    (home-page "https://github.com/simplereach/timeutils")
    ;; Approxidate itself is GPL2.
    (license license:expat)))

;github.com/arduino/go-timeutils
; https://godoc.org/github.com/simplereach/timeutils
;github.com/arduino/go-errors
(define-public go-github-com-arduino-go-properties-map
  (let ((commit "ad37f0cfeff29fadeabe6b2f7f852d8db1fb5c41")
        (revision "0"))
    (package
      (name "go-github-com-arduino-go-properties-map")
      (version (git-version "0.0.0" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                       (url "https://github.com/simplereach/timeutils.git")
                       (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "10ym7qlq77nynmkxbk767f2hfwyxg2k7hrzph05hvgzv843dhivh"))))
      (build-system go-build-system)
      (arguments
       '(#:import-path "github.com/arduino/go-properties-map"))
      (synopsis "FIXME")
      (description "FIXME")
      (home-page "https://github.com/arduino/go-properties-map")
      (license license:expat)))) ; FIXME

(define-public arduino-builder
  (package
    (name "arduino-builder")
    (version "1.4.1")
    (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/arduino/arduino-builder.git")
              (commit version)))
        (sha256
         (base32
          "03z0iqr23a20qj70r3l1cblkzf56qyb8a2zsmjxchjzgqrr2nhbc"))
        (file-name (string-append "arduino-builder-" version))))
    (build-system go-build-system)
    (home-page "https://github.com/arduino/arduino-builder")
    (arguments
     `(#:import-path "github.com/arduino/arduino-builder"))
    (native-inputs
     `(("go-golang-org-x-text-transform" ,go-golang-org-x-text-transform)))
    (synopsis "Command-line tool for compiling Arduino sketches")
    (description "This package is able to parse Arduino Hardware specifications, properly run gcc and produce compiled sketches.")
    (license license:gpl2)))

(define-public arduino
  (package
    (name "arduino")
    (version "1.8.7")
    (source
      (arduino-source version))
    (build-system ant-build-system)
    (arguments
     `(#:jar-name "arduino"
       #:build-target "build"
       #:tests? #f ; No tests exist.
       #:modules ((guix build ant-build-system)
                  (guix build utils)
                  (guix build java-utils)
                  (ice-9 match)
                  (srfi srfi-1))
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'honor-classpath
           (lambda _
             (substitute* "app/build.xml"
              (("classpathref=\"class.path\"")
               "classpath=\"${env.CLASSPATH}\""))
             #t))
         (add-after 'honor-classpath 'disable-apple
           (lambda _
             (substitute* "app/src/processing/app/Base.java"
              (("\\|\\| com\\.apple\\.eawt\\.Application\\.getApplication\\(\\)\\.isAboutMenuItemPresent\\(\\)")
               "")
              (("import processing\\.app\\.macosx\\.ThinkDifferent;")
               "")
              (("ThinkDifferent\\.init\\(\\);")
               ";"))
             (delete-file-recursively "app/src/processing/app/macosx")
             #t))
         (add-before 'build 'chdir
           (lambda* (#:key inputs #:allow-other-keys)
             (install-file (string-append (assoc-ref inputs "arduino-core")
                                          "/share/java/arduino-core.jar")
                           "arduino-core")
             (chdir "app")
             #t))
         (replace 'install
          (lambda* (#:key outputs inputs #:allow-other-keys)
            (let* ((out (assoc-ref outputs "out"))
                   (out-share (string-append out "/share/arduino"))
                   (out-share-lib (string-append out "/share/arduino/lib"))
                   (out-share-part (string-append out-share "/pde.jar"))
                   (out-bin (string-append out "/bin/arduino")))
              (mkdir-p out-share)
              (mkdir-p out-share-lib)
              (for-each (lambda (basename)
                          (install-file (string-append "../build/shared/lib/"
                                        basename)
                                        out-share-lib))
                        '("preferences.txt"
                          "public.gpg.key"
                          "about@2x.png"
                          "arduino.png"
                          "splash.png"
                          "about.png"
                          "formatter.conf"
                          "arduino_icon.ico"
                          "keywords.txt"
                          "splash.bmp"))
              (copy-recursively "../build/shared/lib/theme"
                                (string-append out-share-lib "/theme"))
              (copy-recursively "../build/shared/icons"
                                (string-append out-share-lib "/icons"))
              (copy-recursively "../build/shared/examples"
                                (string-append out-share "/examples"))
              ;; TODO out-share: tools-builder, hardware, reference, libraries, java, tools
              (mkdir-p (string-append out-share "/hardware/arduino"))
              (symlink (string-append (assoc-ref inputs "arduinocore-avr")
                                      "/share/hardware/arduino/avr")
                       (string-append out-share "/hardware/arduino/avr"))
              (symlink (string-append (assoc-ref inputs "java-astyle")
                                      "/lib/libastylej.so")
                       (string-append out-share-lib "/libastylej.so"))
              (mkdir-p (dirname out-bin))
              ;; FIXME: Take the original one (where is it?)
              (invoke "touch"
                      (string-append out-share "/hardware/platform.keys.rewrite.txt"))
              (copy-file "pde.jar" out-share-part)
              ;; FIXME hard-code Java implementation.
              (call-with-output-file out-bin
                (lambda (port)
                  (format port "#!/bin/sh
exec java -DAPP_DIR=\"${APP_DIR}\" processing.app.Base \"$@\"
")))
              (chmod out-bin #o777)
              (wrap-program out-bin
               `("CLASSPATH" ":" suffix ,(append-map
                                          (match-lambda
                                           ((name . dir)
                                            (find-files dir "\\.jar")))
                                          (append inputs outputs)))
               `("APP_DIR" ":" suffix (,out-share)))
              #t))))))
    (inputs
     `(("arduino-core" ,arduino-core)
       ("arduinocore-avr" ,arduinocore-avr)
       ("java-astyle" ,java-astyle)
       ; FIXME: ("java-batik" ,java-batik)
       ("java-rsyntaxtextarea" ,java-rsyntaxtextarea)))
    (home-page "https://www.arduino.cc/")
    (synopsis "Arduino IDE")
    (description "Arduino IDE")
    ;; FIXME.
    (license license:gpl2+)))
